{.-----------------------------------------------------------------------------
  DCTDLNK2 - DelphiCodeToDoc Linker 2 for DCTD 0.8
  This application simply creates links to project members in DCTD documentation
  @author l0co@wp.pl
  @version 2.0<br>File created: 11.10.2004
-----------------------------------------------------------------------------.}

program dctdlnk2;

{$APPTYPE CONSOLE}

uses
  Windows,
  Classes,
  SysUtils,
  uDataClasses in 'uDataClasses.pas',
  uGlobals in 'uGlobals.pas';

const
  CHeader = 'DCTDLNK2 for DCTD 0.8 version %s by l0co@wp.pl';
  CVersion = '2.0';

var
  s: String; //. Temporary string
  i,j: Integer; //. Temporary integers
  t: TDateTime; //. To calculate process time
  tx: Text; //. Text file variable used to create default CSS (+S option)
  srec: TSearchRec; //. Search record for copy other project files (+C option)

{.-----------------------------------------------------------------------------
  Displays info on screen and in debug file
  @param S Text to display
-----------------------------------------------------------------------------.}
procedure DblWrite(S: String = '');
begin
  Writeln(S);
  if Options[COptDebug].Value then Writeln(DebugFile, S);
end;

begin
  Writeln(Format(CHeader, [CVersion]));

  SetLength(TrueBoolStrs, 1);
  TrueBoolStrs[0] := 'on';
  SetLength(FalseBoolStrs, 1);
  FalseBoolStrs[0] := 'off';

  // Check command-line options
  if ParamCount > 0 then
    for i:=1 to ParamCount do begin
      s := ParamStr(i);
      if (s[1] = '-') or (s[1] = '+') then
        for j := 0 to COptCount-1 do
          if (UpperCase(s[2]) = Options[j].Shortcut) then
            Options[j].Value := (s[1] = '+') else
      else
        if SourceDir = '' then SourceDir := s else DestDir := s;
    end;
  if DestDir = '' then DestDir := SourceDir;

  // Display help
  if SourceDir = '' then begin
    Writeln('USAGE: DCTDLNK2 [options] <src_dir> [dest_dir]');
    Writeln;
    Writeln('Directories:');
    Writeln('  src_dir - source directory with DCTD documentation html files');
    Writeln('  dest_dir - destination directory (if not specified will use src_dir)');
    Writeln;
    Writeln('Options format:');
    Writeln('  -X - toggle X option off');
    Writeln('  +X - toggle X option on');
    Writeln;
    Writeln('Options:');
    for j:=0 to COptCount-1 do
      Writeln('  ', Options[j].Shortcut, ' - ', Options[j].Name, ' (default ', BoolToStr(Options[j].Value, true), ')');
    Exit;
  end;

  // Correct '/' character in SourceDir and DestDir and directories exists
  if SourceDir[Length(SourceDir)] <> '\' then SourceDir := SourceDir + '\';
  if DestDir[Length(DestDir)] <> '\' then DestDir := DestDir + '\';
  if not DirectoryExists(SourceDir) then begin
    Writeln;
    Writeln('Source directory not exists:');
    Writeln(SourceDir);
    Exit;
  end;
  if not DirectoryExists(DestDir) then begin
    Writeln;
    Writeln('Destination directory not exists:');
    Writeln(DestDir);
    Exit;
  end;

  // Create debug file if +D option specified
  if Options[COptDebug].Value then begin
    Assign(DebugFile, ExtractFilePath(ParamStr(0)) + CDebugFilename);
    Rewrite(DebugFile);
    Writeln(DebugFile, 'DCTDLNK2 v' + CVersion + ' debug log created at ' + DateTimeToStr(Date) + ' ' + TimeToStr(GetTime));
  end;

  // Display info
  DblWrite;
  DblWrite('Using directories:');
  DblWrite('  From: ' + SourceDir);
  DblWrite('  To: ' + DestDir);
  DblWrite;
  DblWrite('Using options:');
  for j:=0 to COptCount-1 do
    if Options[j].Value then
      DblWrite('  +' + Options[j].Shortcut + ' ' + Options[j].Name);
  DblWrite;

  // Process files
  t := GetTime;

  DblWrite('Reading files list...');
  FileList := TStringList.Create; // Create files list object
  FileList.Sorted := True; // It has to be sorted
  ReadFileList;

  if FileList.Count = 0 then
    DblWrite('No project files found in specified directory.')
  else begin
    IList := TItemList.Create(False);

    DblWrite('Creating objects...');
    FileProject := TFileContainer.Create(FileList.Strings[0], nil); // Creates main project object and descendant objects
    IList.Sort(@CompareFileItems); // Sort list of object names !!! very important - to speed-optimized binary search

    DblWrite('Reading files...');
    FileProject.Read;

    DblWrite('Parsing files...');
    FileProject.Parse;

    DblWrite('Writing parsed files...');
    FileProject.Write;

    DblWrite('Destroying objects...');
    FileProject.Free;
    IList.Free;
    FileList.Free;

    // +C option - copy other project files to destination
    if (SourceDir <> DestDir) and (Options[COptCopy].Value) then begin
      DblWrite('Copying other files...');
      if FindFirst(SourceDir + '*.*', faAnyFile, srec) = 0 then
       begin
         repeat
           s := extractfileext(srec.Name);
           if ((srec.Attr and faDirectory) = 0) and (ExtractFileExt(srec.Name) <> '.' + CFilenameExtension) then
             CopyFile(PChar(SourceDir + srec.Name), PChar(DestDir + srec.Name), False);
         until FindNext(srec) <> 0;
         FindClose(srec);
       end;
    end;

    // +S option - create default css file
    if Options[COptCSS].Value then begin
      DblWrite('Creating default css...');
      AssignFile(tx, DestDir + CCSSFilename);
      Rewrite(tx);
      Write(tx, CCSSFile);
      CloseFile(tx);
    end;

    LongTimeFormat := 's.z';
    DblWrite;
    DblWrite('Finished in ' + TimeToStr(GetTime - t) + 's');
  end;

  // Close debug file
  if Options[COptDebug].Value then CloseFile(DebugFile);
end.
