{.-----------------------------------------------------------------------------
  Globals
  @author l0co@wp.pl
  @version File created: 2004.10.05, 18:53:54
-----------------------------------------------------------------------------.}

unit uGlobals;

interface

uses Classes, StdCtrls, SysUtils;

{------------------------------------------------------------------------------
  Constants
------------------------------------------------------------------------------}

const
  CSeparators: Set of Char = [' ', #8, #10, #13, ',', ';', '"', '?', '[', ']', '!', '@', '#', '$', '%', '^', '&' ,'*', '-', ':', '{', '}', '(', ')', '\', '|', '/', '~', '`', '''']; //. Words' separators
  CFilenameSeparator = '_'; //. Sections' separator in file name
  CFilenameExtension = 'htm'; //. Extension of project file names
  CLastSection = -1; //. Last section in GetFilenameSection calls
  CDebugFilename = 'debug.log'; //. File name to debug log

  CInheritanceText_PL = 'Hierarchia klas'; //. Polish version of 'class hierarchy' string in documentation
  CInheritanceText_EN = 'Class Hierarchy'; //. English version
  //. String to search for inheritance in file
  CInheritance =
    '<span class="SectionName">' + CInheritanceText_EN + '</span><br>' + #13 + #10 +
    '<table width="95%" border="0" cellspacing="0">' + #13 + #10 +
    '  <tr>' + #13 + #10 +
    '    <td width="10">&nbsp;</td>' + #13 + #10 +
    '    <td class="Generated">';

  //. Default CSS file to include in destination dir
  CCSSFile =
    'a {text-decoration: none}' + #13 + #10 +
    'a:link{color:#2020A0}' + #13 + #10 +
    'a:visited{color:#2020A0}' + #13 + #10 +
    'a:hover{color:#2020A0}' + #13 + #10 +
    'a:hover {color:#2020F0; text-decoration: underline}' + #13 + #10 +
    '.SectionName {  font-weight: bold; color: #000000; font-family: Arial, Verdana, Helvetica, sans-serif; font-size: x-small}' + #13 + #10 +
    '.Generated {  color: #000000; font-family: Arial, Verdana, Helvetica, sans-serif; font-size: x-small; font-style: normal; font-weight: normal}' + #13 + #10 +
    '.IndexName { font-weight: normal; background-color: #CCCCCC; font-family: Arial, Verdana, Helvetica, sans-serif; font-size: x-small}' + #13 + #10 +
    '.Normal {  font-family: Arial, Verdana, Helvetica, sans-serif; font-size: x-small; font-style: normal; font-weight: normal}' + #13 + #10 +
    '.Footer { color: #000000; font-family: Arial, Verdana, Helvetica, sans-serif; font-size: xx-small; font-style: normal; font-weight: normal }';
  CCSSFileName = 'DelphiCodeToDoc.css'; //. Default CSS file name

  CLinkFormat = '<a href="%s" title="%s">%s</a>'; //. Links format (filename, hint, text)

{------------------------------------------------------------------------------
  Options
------------------------------------------------------------------------------}

type
  TOption = record //. Command-line single option structure
    Name: String[60];
    Shortcut: Char;
    Value: Boolean;
  end;

const
  COptCount = 4; //. Count of options
  COptDebug = 0; //. Command-line option: Debug log
  COptUncertain = 1; //. Command-line option: Create uncertain links
  COptCopy = 2; //. Command-line option: Copy other (non-HTML) files from source to destination
  COptCSS = 3; //. Command-line option: Include default CSS file (from CCSSFile constant)


var
  //. Command-line options array
  Options: array[0..COptCount-1] of TOption = ( (Name: 'log debug to debug.log'; Shortcut: 'D'; Value: False),
                                      (Name: 'create uncertain links'; Shortcut: 'L'; Value: False),
                                      (Name: 'copy other (non-html) files from source to destination'; Shortcut: 'C'; Value: True),
                                      (Name: 'create default CSS file'; Shortcut: 'S'; Value: False) );

{------------------------------------------------------------------------------
  Variables
------------------------------------------------------------------------------}

var
  DebugFile: Text; //. Log file to debug
  FileList: TStringList; //. List of project files
  SourceDir: String = ''; //. Source directory
  DestDir: String = ''; //. Destination directory
  FileMask: String = '*'; //. File name mask to parse (on name part, no file extension)
  ListPosition: Integer = 0; //. Temporary, used in TFileItem constructor

{------------------------------------------------------------------------------
  Function declarations
------------------------------------------------------------------------------}

procedure ReadFileList;
function GetSectionCount(AFilename: String): Integer;
function GetFilenameSection(AFilename: String; ASection: Integer): String;
function HasSubItem(Index: Integer): Boolean;

{------------------------------------------------------------------------------
  Implementation
------------------------------------------------------------------------------}

implementation

{.-----------------------------------------------------------------------------
  Reads list of project files to FileList
  Don't creates FileList - it has to be created elsewhere
-----------------------------------------------------------------------------.}
procedure ReadFileList;
var
  sr: TSearchRec;
begin
  if FindFirst(SourceDir + FileMask + '.' + CFilenameExtension, faAnyFile, sr) = 0 then
   begin
     repeat
       if (sr.Attr and faDirectory) = 0 then
         FileList.Add(Copy(sr.Name, 1, Pos(ExtractFileExt(sr.Name), sr.Name)-1));
     until FindNext(sr) <> 0;
     FindClose(sr);
   end;
end;

{.-----------------------------------------------------------------------------
  Returns count of sections in file name
  @param AFilename File name
  @return Count of sections, by CFilenameSeparator as delimeter
-----------------------------------------------------------------------------.}
function GetSectionCount(AFilename: String): Integer;
begin
  Result := 1;
  while (Pos(CFilenameSeparator, AFilename)>0) and (Result<4) do begin
    AFilename := Copy(AFilename, Pos(CFilenameSeparator, AFilename)+1, Length(AFilename));
    Inc(Result);
  end;
  if Result > 4 then Result := 4;
end;

{.-----------------------------------------------------------------------------
  Returns gived section form file name
  If there's more than 4 sections, returns all sections after 4 as one
  @param AFilename File name
  @param ASection Section number or CLastSection
  @return Section of file name
-----------------------------------------------------------------------------.}
function GetFilenameSection(AFilename: String; ASection: Integer): String;
var
  Index: Integer;
begin
  Result := AFilename;
  if ASection = CLastSection then ASection := GetSectionCount(AFilename);
  Index := ASection;

  while Index > 1 do begin
    if Pos(CFilenameSeparator, Result) > 0 then
      Delete(Result, 1, Pos(CFilenameSeparator, Result)) else
      Delete(Result, 1, Length(Result));
    Dec(Index);
  end;

  if (Pos(CFilenameSeparator, Result) > 0) and (ASection < 4) then
    Delete(Result, Pos(CFilenameSeparator, Result), Length(Result));
end;

{.-----------------------------------------------------------------------------
  Checks the file name has subitems
  @param Index Index of file in FileList
  @return TRUE if file has subitems
  @return FALSE otherwise
-----------------------------------------------------------------------------.}
function HasSubItem(Index: Integer): Boolean;
begin
  Result := False;
  if Index >= FileList.Count-1 then Exit;
  if Pos(FileList.Strings[Index] + CFilenameSeparator, FileList.Strings[Index+1]) <> 0 then Result := True;
end;

end.
